@@ -8,12 +8,12 @@ use Carp ();
 
 #use Smart::Comments '####';
 
-our $VERSION  = '1.02';
+our $VERSION  = '1.03';
 
 # Regexp for HTML tags
 
 my $SPACE       =  q{\s};
-my $IDENT       =  q{[a-zA-Z]+};
+my $IDENT       =  q{\w+};
 my $ATTR_VALUE  =  q{(?: " [^"]* " | ' [^']* ' | [^'"/>/\s]+ )};
 my $ATTR        = qq{(?:$SPACE+ $IDENT = $ATTR_VALUE )};
 
@@ -45,7 +45,32 @@ my $MULTIPLE = q{(?: multiple = (?: "multiple" | 'multiple' | multiple ) )};
 #}
 
 
-sub new{
+# utilities for getting HTML attributes
+sub _unquote{
+	$_[0] =~ /(['"]) (.*) \1/xms ? $2 : $_[0];
+}
+sub _get_id{
+	$_[0] =~ /id    = ($ATTR_VALUE)/oxmsi ? _unquote($1) : undef;
+}
+sub _get_type{
+	$_[0] =~ /type  = ($ATTR_VALUE)/oxmsi ? _unquote($1) : undef;
+}
+sub _get_name{
+	$_[0] =~ /name  = ($ATTR_VALUE)/oxmsi ? _unquote($1) : undef;
+}
+sub _get_value{
+	$_[0] =~ /value = ($ATTR_VALUE)/oxmsi ? _unquote($1) : undef;
+}
+
+#use macro
+#	_unquote   => \&_unquote,
+#	_get_id    => \&_get_id,
+#	_get_type  => \&_get_type,
+#	_get_name  => \&_get_name,
+#	_get_value => \&_get_value,
+#;
+
+sub new :method{
 	my $class = shift;
 	return $class->_parse_option(@_);
 }
@@ -67,10 +92,12 @@ sub _parse_option{
 			file     => 1,
 		},
 
-		escape      => \&_escape_html,
+		escape        => \&_escape_html,
+		decode_entity => \&_noop,
+		layer         => '',
 	);
 
-	# merge
+	# merge if needed
 	foreach my $key( ref($self) ? keys %{$self} : () ){
 		$ctx{$key} = ref($self->{$key}) eq 'HASH'
 			? { %{$self->{$key}} } : $self->{$key};
@@ -101,6 +128,9 @@ sub _parse_option{
 				$ctx{escape} = \&_noop;
 			}
 		}
+		elsif($opt eq 'layer'){
+			$ctx{layer} = $val;
+		}
 		elsif($opt eq 'decode_entity'){
 			if($val){
 				$ctx{decode_entity} = ref($val) eq 'CODE'
@@ -108,7 +138,7 @@ sub _parse_option{
 					: \&_decode_entity;
 			}
 			else{
-				delete $ctx{decode_entity};
+				$ctx{decode_entity} = \&_noop;
 			}
 		}
 		else{
@@ -119,7 +149,7 @@ sub _parse_option{
 	return bless \%ctx => ref($self) || $self;
 }
 
-sub fill{
+sub fill :method{
 	my($self, $src, $q, @opt) = @_;
 
 	defined $src or Carp::croak('No source supplied');
@@ -139,11 +169,11 @@ sub fill{
 	}
 	else{
 		if(not defined fileno $src){
-			open my($in), '<', $src
+			open my($in), '<'.$ctx->{layer}, $src
 				or Carp::croak("Cannot open '$src': $!");
 			$src = $in;
 		}
-		$content = do{ local $/; <$src> };
+		$content = do{ local $/; <$src> }; # slurp
 	}
 
 	# if $content is utf8-flagged, params should be utf8-encoded
@@ -159,13 +189,15 @@ sub fill{
 	if(defined $ctx->{target}){
 
 		$content =~ s{ ($FORM) (.*?) ($END_FORM) }
-		             {	my($form, $content, $end_form) = ($1, $2, $3);
+		             {
+				my($beg, $content, $end) = ($1, $2, $3);
 
-				my $id = _get_id($form);
+				my $id = _get_id($beg);
 				(defined($id) and $ctx->{target} eq $id)
-					? $form . _fill($ctx, $content) . $end_form
-					: $form . $content . $end_form
+					? $beg . _fill($ctx, $content) . $end
+					: $beg .             $content  . $end
 			     }goexmsi;
+
 		return $content;
 	}
 	else{
@@ -176,7 +208,8 @@ sub fill{
 
 sub _fill{
 	my($ctx, $content) = @_;
-	$content =~ s{($INPUT)}{ _fill_input($ctx, $1)        }goexmsi;
+	$content =~ s{($INPUT)}
+		     { _fill_input($ctx, $1)                  }goexmsi;
 
 	$content =~ s{($SELECT) (.*?) ($END_SELECT) }
 		     { $1 . _fill_select($ctx, $1, $2) . $3   }goexmsi;
@@ -187,6 +220,7 @@ sub _fill{
 	return $content;
 }
 
+
 sub _fill_input{
 	my($ctx, $tag) = @_;
 
@@ -206,7 +240,7 @@ sub _fill_input{
 		if(not defined $value){
 			$value = 'on';
 		}
-		elsif($ctx->{decode_entity}){
+		else{
 			$value = $ctx->{decode_entity}->($value);
 		}
 
@@ -253,9 +287,7 @@ sub _fill_option{
 		$value =~ s{   $SPACE+ \z} {}oxms;
 	}
 
-	if($ctx->{decode_entity}){
-		$value = $ctx->{decode_entity}->($value);
-	}
+	$value = $ctx->{decode_entity}->($value);
 
 	### @_
 	if(grep{ $value eq $_ }  @{$values_ref}){
@@ -293,8 +325,7 @@ sub _get_param{
 
 		if($ctx->{utf8}){
 			for my $datum( @{$ref} ){
-				utf8::decode($datum)
-					unless utf8::is_utf8($datum);
+				utf8::decode($datum) unless utf8::is_utf8($datum);
 			}
 		}
 	}
@@ -315,39 +346,26 @@ sub _escape_html{
 	$s =~ s/"/&quot;/g;
 	return $s;
 }
+
+
 sub _decode_entity{
 	my $s = shift;
 
-	if($s =~ /&\w+;/){
-		require HTML::Entities;
-		return HTML::Entities::decode($s);
+	our %entity2char;
+	unless(%entity2char){
+		# load the HTML entity data
+		local $/ = "__END__\n";
+		local($@, $!);
+		open my $data_in, '<', __FILE__ or die $!; # should be success
+		readline $data_in; # discard the first segment
+		eval scalar <$data_in> or die $@;
 	}
-	else{
-		$s =~ s{&\#(\d+)          ;}{ chr     $1 }egxms;
-		$s =~ s{&\#x([0-9a-fA-F]+);}{ chr hex $1 }egxms;
-		return $s;
-	}
-}
 
-sub _unquote{
-	$_[0] =~ /(['"]) (.*) \1/xms or return $_[0];
-	return $2;
-}
-sub _get_id{
-	$_[0] =~ /id    = ($ATTR_VALUE)/oxmsi or return;
-	return _unquote($1);
-}
-sub _get_type{
-	$_[0] =~ /type  = ($ATTR_VALUE)/oxmsi or return;
-	return _unquote($1);
-}
-sub _get_name{
-	$_[0] =~ /name  = ($ATTR_VALUE)/oxmsi or return;
-	return _unquote($1);
-}
-sub _get_value{
-	$_[0] =~ /value = ($ATTR_VALUE)/oxmsi or return;
-	return _unquote($1);
+	$s =~ s{&(\w+);}{ $entity2char{$1} || "&$1;" }egxms;
+
+	$s =~ s{&\#(\d+)          ;}{ chr     $1 }egxms;
+	$s =~ s{&\#x([0-9a-fA-F]+);}{ chr hex $1 }egxms;
+	return $s;
 }
 
 #sub _disable{
@@ -379,7 +397,7 @@ sub _to_form_object{
 				ref($_) eq 'ARRAY' ?  $_  :
 				defined($_)        ? [$_] :
 						      ();
-			     } values %{$ref};
+			} values %{$ref};
 	}
 	elsif($type eq 'ARRAY'){
 		$wrapper = [];
@@ -433,6 +451,147 @@ sub HTML::FillInForm::Lite::Object::param{
 
 __END__
 
+our %entity2char = (
+	quot   => chr(34),
+	amp    => chr(38),
+	apos   => chr(39),
+	lt     => chr(60),
+	gt     => chr(62),
+
+	AElig	=> chr(198),	Aacute	=> chr(193),
+	Acirc	=> chr(194),	Agrave	=> chr(192),
+	Aring	=> chr(197),	Atilde	=> chr(195),
+	Auml	=> chr(196),	Ccedil	=> chr(199),
+	ETH		=> chr(208),	Eacute	=> chr(201),
+	Ecirc	=> chr(202),	Egrave	=> chr(200),
+	Euml	=> chr(203),	Iacute	=> chr(205),
+	Icirc	=> chr(206),	Igrave	=> chr(204),
+	Iuml	=> chr(207),	Ntilde	=> chr(209),
+	Oacute	=> chr(211),	Ocirc	=> chr(212),
+	Ograve	=> chr(210),	Oslash	=> chr(216),
+	Otilde	=> chr(213),	Ouml	=> chr(214),
+	THORN	=> chr(222),	Uacute	=> chr(218),
+	Ucirc	=> chr(219),	Ugrave	=> chr(217),
+	Uuml	=> chr(220),	Yacute	=> chr(221),
+	aacute	=> chr(225),	acirc	=> chr(226),
+	aelig	=> chr(230),	agrave	=> chr(224),
+	aring	=> chr(229),	atilde	=> chr(227),
+	auml	=> chr(228),	ccedil	=> chr(231),
+	eacute	=> chr(233),	ecirc	=> chr(234),
+	egrave	=> chr(232),	eth		=> chr(240),
+	euml	=> chr(235),	iacute	=> chr(237),
+	icirc	=> chr(238),	igrave	=> chr(236),
+	iuml	=> chr(239),	ntilde	=> chr(241),
+	oacute	=> chr(243),	ocirc	=> chr(244),
+	ograve	=> chr(242),	oslash	=> chr(248),
+	otilde	=> chr(245),	ouml	=> chr(246),
+	szlig	=> chr(223),	thorn	=> chr(254),
+	uacute	=> chr(250),	ucirc	=> chr(251),
+	ugrave	=> chr(249),	uuml	=> chr(252),
+	yacute	=> chr(253),	yuml	=> chr(255),
+
+	copy   => chr(169),	reg    => chr(174),
+	nbsp   => chr(160),
+
+	iexcl  => chr(161),	cent   => chr(162),
+	pound  => chr(163),	curren => chr(164),
+	yen    => chr(165),	brvbar => chr(166),
+	sect   => chr(167),	uml    => chr(168),
+	ordf   => chr(170),	laquo  => chr(171),
+	not    => chr(172),	shy    => chr(173),
+	macr   => chr(175),	deg    => chr(176),
+	plusmn => chr(177),	sup1   => chr(185),
+	sup2   => chr(178),	sup3   => chr(179),
+	acute  => chr(180),	micro  => chr(181),
+	para   => chr(182),	middot => chr(183),
+	cedil  => chr(184),	ordm   => chr(186),
+	raquo  => chr(187),	frac14 => chr(188),
+	frac12 => chr(189),	frac34 => chr(190),
+	iquest => chr(191),	times  => chr(215),
+	divide => chr(247),
+
+	OElig    => chr(338),	oelig    => chr(339),
+	Scaron   => chr(352),	scaron   => chr(353),
+	Yuml     => chr(376),	fnof     => chr(402),
+	circ     => chr(710),	tilde    => chr(732),
+	Alpha    => chr(913),	Beta     => chr(914),
+	Gamma    => chr(915),	Delta    => chr(916),
+	Epsilon  => chr(917),	Zeta     => chr(918),
+	Eta      => chr(919),	Theta    => chr(920),
+	Iota     => chr(921),	Kappa    => chr(922),
+	Lambda   => chr(923),	Mu       => chr(924),
+	Nu       => chr(925),	Xi       => chr(926),
+	Omicron  => chr(927),	Pi       => chr(928),
+	Rho      => chr(929),	Sigma    => chr(931),
+	Tau      => chr(932),	Upsilon  => chr(933),
+	Phi      => chr(934),	Chi      => chr(935),
+	Psi      => chr(936),	Omega    => chr(937),
+	alpha    => chr(945),	beta     => chr(946),
+	gamma    => chr(947),	delta    => chr(948),
+	epsilon  => chr(949),	zeta     => chr(950),
+	eta      => chr(951),	theta    => chr(952),
+	iota     => chr(953),	kappa    => chr(954),
+	lambda   => chr(955),	mu       => chr(956),
+	nu       => chr(957),	xi       => chr(958),
+	omicron  => chr(959),	pi       => chr(960),
+	rho      => chr(961),	sigmaf   => chr(962),
+	sigma    => chr(963),	tau      => chr(964),
+	upsilon  => chr(965),	phi      => chr(966),
+	chi      => chr(967),	psi      => chr(968),
+	omega    => chr(969),	thetasym => chr(977),
+	upsih    => chr(978),	piv      => chr(982),
+
+	ensp     => chr(8194),	emsp     => chr(8195),
+	thinsp   => chr(8201),	zwnj     => chr(8204),
+	zwj      => chr(8205),	lrm      => chr(8206),
+	rlm      => chr(8207),	ndash    => chr(8211),
+	mdash    => chr(8212),	lsquo    => chr(8216),
+	rsquo    => chr(8217),	sbquo    => chr(8218),
+	ldquo    => chr(8220),	rdquo    => chr(8221),
+	bdquo    => chr(8222),	dagger   => chr(8224),
+	Dagger   => chr(8225),	bull     => chr(8226),
+	hellip   => chr(8230),	permil   => chr(8240),
+	prime    => chr(8242),	Prime    => chr(8243),
+	lsaquo   => chr(8249),	rsaquo   => chr(8250),
+	oline    => chr(8254),	frasl    => chr(8260),
+	euro     => chr(8364),	image    => chr(8465),
+	weierp   => chr(8472),	real     => chr(8476),
+	trade    => chr(8482),	alefsym  => chr(8501),
+	larr     => chr(8592),	uarr     => chr(8593),
+	rarr     => chr(8594),	darr     => chr(8595),
+	harr     => chr(8596),	crarr    => chr(8629),
+	lArr     => chr(8656),	uArr     => chr(8657),
+	rArr     => chr(8658),	dArr     => chr(8659),
+	hArr     => chr(8660),	forall   => chr(8704),
+	part     => chr(8706),	exist    => chr(8707),
+	empty    => chr(8709),	nabla    => chr(8711),
+	isin     => chr(8712),	notin    => chr(8713),
+	ni       => chr(8715),	prod     => chr(8719),
+	sum      => chr(8721),	minus    => chr(8722),
+	lowast   => chr(8727),	radic    => chr(8730),
+	prop     => chr(8733),	infin    => chr(8734),
+	ang      => chr(8736),	and      => chr(8743),
+	or       => chr(8744),	cap      => chr(8745),
+	cup      => chr(8746),	int      => chr(8747),
+	there4   => chr(8756),	sim      => chr(8764),
+	cong     => chr(8773),	asymp    => chr(8776),
+	ne       => chr(8800),	equiv    => chr(8801),
+	le       => chr(8804),	ge       => chr(8805),
+	sub      => chr(8834),	sup      => chr(8835),
+	nsub     => chr(8836),	sube     => chr(8838),
+	supe     => chr(8839),	oplus    => chr(8853),
+	otimes   => chr(8855),	perp     => chr(8869),
+	sdot     => chr(8901),	lceil    => chr(8968),
+	rceil    => chr(8969),	lfloor   => chr(8970),
+	rfloor   => chr(8971),	lang     => chr(9001),
+	rang     => chr(9002),	loz      => chr(9674),
+	spades   => chr(9824),	clubs    => chr(9827),
+	hearts   => chr(9829),	diams    => chr(9830),
+);
+1;
+
+__END__
+
 =encoding UTF-8
 
 =head1 NAME
@@ -441,7 +600,7 @@ HTML::FillInForm::Lite - Fills in HTML forms with data
 
 =head1 VERSION
 
-The document describes HTML::FillInForm::Lite version 1.02
+The document describes HTML::FillInForm::Lite version 1.03
 
 =head1 SYNOPSIS
 
@@ -521,15 +680,31 @@ Note that it is not implemented in C<HTML::FillInForm>.
 
 =item decode_entity => I<bool> | I<ref>
 
-If true is provided, HTML entities in state fields (namely, radio, checkbox
-and select) will be decoded, but normally it is not needed.
+If true is provided , HTML entities in state fields
+(namely, C<radio>, C<checkbox> and C<select>) will be decoded,
+but normally it is not needed.
 
 You can also supply a subroutine reference to decode HTML entities.
 
-If there are named entities in the fields and the option is true,
-C<HTML::Entities> will be required.
+Note that C<HTML::FillInForm> always decodes HTML entities in state fields,
+but not supports this option.
 
-Note that it is default in C<HTML::FillInForm>.
+=item layer => I<:iolayer>
+
+To read a file with I<:iolayer>. It is used when a file name is supplied as
+I<source>.
+
+For example:
+
+	# To read a file encoded in UTF-8
+	$fif = HTML::FillInForm::Lite->new(layer => ':utf8');
+	$output = $fif->fill($file, $fdat);
+
+	# To read a file encoded in EUC-JP
+	$fif = HTML::FillInForm::Lite->new(layer => ':encoding(euc-jp)');
+	$output = $fif->fill($file, $fdat);
+
+Note that it is not implemented in C<HTML::FillInForm>.
 
 =back
 
@@ -541,18 +716,25 @@ supplied, it will cause C<die>.
 I<options> are the same as C<new()>'s.
 
 You can use this method as a both class or instance method, 
-but you make multiple calls to C<fill()> with the same
-options, it is a little faster to call C<new()> before C<fill()>.
+but you make multiple calls to C<fill()> with the B<same>
+options, it is a little faster to call C<new()> and store the instance.
 
-If I<form_data> is a subroutine reference, it will be called in list context.
-That is, to leave some fields untouched, it must return C<()>, not C<undef>.
+I<source> may be a scalar reference, an array reference of strings, or
+a file name.
+
+I<form_data> may be a hash reference, an object with C<param()> method,
+an object with accessors, a code reference, or an array reference of
+those above mentioned.
+
+If I<form_data> is based on procedures (i.e. not a hash reference),
+the procedures will be called in the list context.
+Therefore, to leave some fields untouched, it must return a null list C<()>,
+not C<undef>.
 
 =head1 DEPENDENCIES
 
 Perl 5.8.1 or later.
 
-If you use the C<decode_entity> option, C<HTML::Entities> may be required.
-
 =head1 NOTES
 
 =head2 Compatibility with C<HTML::FillInForm>
@@ -596,7 +778,7 @@ It may cause problems. Suppose there is a code like:
 
 	<script> document.write("<input name='foo' />") </script>
 
-HTML::FillInForm::Lite will process the code to be broken:
+C<HTML::FillInForm::Lite> will break the code:
 
 	<script> document.write("<input name='foo' value="bar" />") </script>
 
@@ -607,7 +789,7 @@ To avoid such problems, you can use the C<ignore_fields> option.
 No bugs have been reported.
 
 Please report any bug or feature request to E<lt>gfuji(at)cpan.orgE<gt>,
-or through L<http://rt.cpan.org/>.
+or through the RT L<http://rt.cpan.org/>.
 
 =head1 SEE ALSO
 
